"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.extendContext = void 0;
const utils_1 = require("./utils");
function extendContext(context, extension, overrideProperties, logger) {
    const descriptors = new Map();
    const skippedKeys = [];
    const previousDescriptors = new Map();
    for (const [key, get] of extension) {
        let shouldSkip = overrideProperties
            ? context.hasOwnProperty(key)
            : key in context;
        if (shouldSkip && overrideProperties) {
            const descriptor = Object.getOwnPropertyDescriptor(context, key);
            if (descriptor?.configurable) {
                previousDescriptors.set(key, descriptor);
                shouldSkip = false;
            }
        }
        if (shouldSkip) {
            skippedKeys.push(key);
            continue;
        }
        descriptors.set(key, {
            get,
            set(value) {
                delete context[key];
                context[key] = value;
            },
            configurable: true,
            enumerable: true,
        });
    }
    for (const [key, descriptor] of descriptors) {
        Object.defineProperty(context, key, descriptor);
    }
    if (skippedKeys.length > 0) {
        logger.warn?.('The following context properties ' +
            (overrideProperties ? 'are not configurable' : 'already exist') +
            ` and has NOT been overriden: ${(0, utils_1.logKeys)(skippedKeys)}.`);
    }
    if (previousDescriptors.size > 0) {
        logger.warn?.(`The following context properties has been overriden: ${(0, utils_1.logKeys)(previousDescriptors.keys())}.`);
    }
    return () => {
        const unrestoredKeys = [];
        for (const [key, { get }] of descriptors) {
            // Restore or delete property only if it was not changed.
            if (get !== Object.getOwnPropertyDescriptor(context, key)?.get) {
                if (previousDescriptors.has(key)) {
                    unrestoredKeys.push(key);
                }
                continue;
            }
            if (previousDescriptors.has(key)) {
                Object.defineProperty(context, key, previousDescriptors.get(key));
            }
            else {
                delete context[key];
            }
        }
        descriptors.clear();
        previousDescriptors.clear();
        if (unrestoredKeys.length > 0) {
            logger.warn?.('The following context properties has not been restored because ' +
                `they were changed during debugging: ${(0, utils_1.logKeys)(unrestoredKeys)}.`);
        }
    };
}
exports.extendContext = extendContext;
